Package: symTensor
Type: Package
Title: Symmetric Matrix and Tensor Decomposition
Version: 0.1.0
Authors@R: person("Koki", "Tsuyuzaki", email = "k.t.the-answer@hotmail.co.jp",
    role = c("aut", "cre"))
Description: Provides symmetric matrix and tensor operations and decomposition
    algorithms including symmetric NMF (symNMF), PageRank, Label Propagation,
    Higher-order Power Method, and TOPHITS. Designed to work with 'rTensor'
    objects. Methods are described in Kuang et al. (2012)
    <doi:10.1137/1.9781611972825.10>, Kolda and Mayo (2011)
    <doi:10.1137/100801482>, and Kolda and Bader (2006)
    <doi:10.1137/1.9781611972764.26>.
License: MIT + file LICENSE
Encoding: UTF-8
Depends: R (>= 3.5.0)
Imports: rTensor, methods, stats
Suggests: testthat
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2026-05-11 07:12:29 UTC; koki
Author: Koki Tsuyuzaki [aut, cre]
Maintainer: Koki Tsuyuzaki <k.t.the-answer@hotmail.co.jp>
Repository: CRAN
Date/Publication: 2026-05-14 09:50:02 UTC
Built: R 4.6.0; ; 2026-05-14 09:59:52 UTC; unix

Package: symTensor
Type: Package
Title: Symmetric Matrix and Tensor Decomposition
Version: 0.1.0
Authors@R: person("Koki", "Tsuyuzaki", email = "k.t.the-answer@hotmail.co.jp",
    role = c("aut", "cre"))
Description: Provides symmetric matrix and tensor operations and decomposition
    algorithms including symmetric NMF (symNMF), PageRank, Label Propagation,
    Higher-order Power Method, and TOPHITS. Designed to work with 'rTensor'
    objects. Methods are described in Kuang et al. (2012)
    <doi:10.1137/1.9781611972825.10>, Kolda and Mayo (2011)
    <doi:10.1137/100801482>, and Kolda and Bader (2006)
    <doi:10.1137/1.9781611972764.26>.
License: MIT + file LICENSE
Encoding: UTF-8
Depends: R (>= 3.5.0)
Imports: rTensor, methods, stats
Suggests: testthat
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2026-05-11 07:12:29 UTC; koki
Author: Koki Tsuyuzaki [aut, cre]
Maintainer: Koki Tsuyuzaki <k.t.the-answer@hotmail.co.jp>
Repository: CRAN
Date/Publication: 2026-05-14 09:50:02 UTC
Built: R 4.6.0; ; 2026-05-14 12:36:41 UTC; unix

# Generated by roxygen2: do not edit by hand

export(HigherOrderPower)
export(LabelPropagation)
export(PageRank)
export(Symmetrize)
export(TOPHITS)
export(isSymmetricTensor)
export(symNMF)
import(methods)
import(rTensor)
importFrom(stats,rnorm)
importFrom(stats,runif)

#' Symmetric Non-negative Matrix Factorization (symNMF)
#'
#' Decomposes a symmetric non-negative matrix \eqn{\Omega \approx M S M^\top}
#' using multiplicative update (MU) rules with Beta-divergence.
#'
#' Two models are supported:
#' \itemize{
#'   \item \strong{MSM}: \eqn{\Omega \approx M S M^\top} where M is N x K and S is K x K
#'   \item \strong{MM}: \eqn{\Omega \approx M M^\top} where M is N x K (special case with S = I)
#' }
#'
#' The loss function is the Beta-divergence \eqn{D_\beta(\Omega \| \hat{\Omega})}:
#' \itemize{
#'   \item \eqn{\beta = 2}: Frobenius (Euclidean) distance
#'   \item \eqn{\beta = 1}: Kullback-Leibler divergence
#'   \item \eqn{\beta = 0}: Itakura-Saito divergence
#'   \item Other values of \eqn{\beta} interpolate/extrapolate these cases
#' }
#'
#' @param Omega Symmetric non-negative matrix (N x N)
#' @param K Number of components (rank)
#' @param model Character, either \code{"MSM"} (default) or \code{"MM"}
#' @param Beta Beta parameter for Beta-divergence (default: 2).
#'   2 = Frobenius, 1 = KL, 0 = IS.
#' @param initM Initial M matrix (N x K). If NULL, random non-negative initialization.
#' @param initS Initial S matrix (K x K). If NULL, random non-negative initialization.
#'   Ignored when \code{model = "MM"}.
#' @param lambda Regularization parameter for \eqn{|\det(S)|} penalty (default: 0).
#'   Only used with \code{model = "MSM"}.
#' @param num.iter Maximum number of iterations (default: 100)
#' @param thr Convergence threshold on relative change (default: 1e-10)
#' @param verbose Logical, print iteration info (default: FALSE)
#' @return A list with components:
#'   \describe{
#'     \item{M}{Factor matrix (N x K)}
#'     \item{S}{Coefficient matrix (K x K). Identity matrix when \code{model = "MM"}.}
#'     \item{RecError}{Vector of reconstruction errors at each iteration}
#'     \item{RelChange}{Vector of relative changes at each iteration}
#'     \item{converged}{Logical}
#'     \item{iter}{Number of iterations performed}
#'   }
#' @references
#' Kuang, D., Park, H., & Ding, C. H. Q. (2012).
#' Symmetric Nonnegative Matrix Factorization for Graph Clustering.
#' SDM 2012.
#'
#' Fevotte, C. & Idier, J. (2011).
#' Algorithms for Nonnegative Matrix Factorization with the
#' Beta-Divergence. Neural Computation, 23(9).
#' @export
#' @examples
#' set.seed(42)
#' M_true <- matrix(c(rep(c(1,0), 5), rep(c(0,1), 5)), 10, 2)
#' Omega <- M_true %*% t(M_true) + matrix(runif(100, 0, 0.1), 10, 10)
#' Omega <- (Omega + t(Omega)) / 2
#'
#' # Frobenius (Beta=2, default)
#' res_fro <- symNMF(Omega, K = 2, Beta = 2)
#'
#' # KL divergence (Beta=1)
#' res_kl <- symNMF(Omega, K = 2, Beta = 1)
#'
#' # Itakura-Saito (Beta=0)
#' res_is <- symNMF(Omega, K = 2, Beta = 0)
symNMF <- function(Omega, K, model = c("MSM", "MM"),
                   Beta = 2,
                   initM = NULL, initS = NULL,
                   lambda = 0,
                   num.iter = 100L, thr = 1e-10,
                   verbose = FALSE) {
  model <- match.arg(model)
  N <- nrow(Omega)
  eps <- .Machine$double.eps

  # Initialize M
  if (is.null(initM)) {
    M <- matrix(runif(N * K, 0, 1), N, K)
  } else {
    M <- initM
  }

  # Initialize S
  if (model == "MSM") {
    if (is.null(initS)) {
      S <- matrix(runif(K * K, 0, 1), K, K)
      S <- (S + t(S)) / 2
    } else {
      S <- initS
    }
  } else {
    S <- diag(K)
  }

  RecError <- numeric(num.iter)
  RelChange <- numeric(num.iter)
  converged <- FALSE

  for (iter in seq_len(num.iter)) {
    # Reconstruction
    Approx <- M %*% S %*% t(M)
    Approx_safe <- Approx + eps

    # Compute Beta-divergence
    RecError[iter] <- .betaDiv(Omega, Approx_safe, Beta)

    # Relative change
    if (iter > 1) {
      RelChange[iter] <- abs(RecError[iter - 1] - RecError[iter]) /
        (abs(RecError[iter]) + eps)
      if (RelChange[iter] < thr) {
        converged <- TRUE
        if (verbose) message("Converged at iteration ", iter)
        break
      }
    }

    if (verbose && (iter %% 10 == 0 || iter == 1)) {
      message("Iter ", iter, ": RecError = ", round(RecError[iter], 6))
    }

    # General Beta-divergence MU updates
    # Weight matrix: W = Omega^(Beta-2) * Approx^(-(Beta-2)) for numerator part
    # Numerator weight: Omega^(Beta-1) * Approx^(-(Beta-1)) = (Omega/Approx)^(Beta-1)
    # Denominator weight: Approx^(Beta-1) * Approx^(-(Beta-1)) = 1 (but with structure)
    #
    # General MU rule for V in X ≈ W H:
    #   H <- H * ((W^T (X * Approx^(Beta-2))) / (W^T Approx^(Beta-1)))^gamma
    # where gamma = 1 for Beta in [1,2], specific exponents otherwise
    #
    # For symmetric case Omega ≈ M S M^T:
    numer_weight <- Omega * Approx_safe^(Beta - 2)
    denom_weight <- Approx_safe^(Beta - 1)

    if (model == "MSM") {
      # Update M
      numer_M <- numer_weight %*% M %*% S
      denom_M <- denom_weight %*% M %*% S + eps
      gamma <- .muExponent(Beta)
      M <- M * (numer_M / denom_M)^gamma

      # Recompute Approx after M update
      Approx <- M %*% S %*% t(M)
      Approx_safe <- Approx + eps
      numer_weight <- Omega * Approx_safe^(Beta - 2)
      denom_weight <- Approx_safe^(Beta - 1)

      # Update S
      numer_S <- t(M) %*% numer_weight %*% M
      denom_S <- t(M) %*% denom_weight %*% M + eps
      if (lambda > 0) {
        S_inv <- tryCatch(solve(S + eps * diag(K)), error = function(e) NULL)
        if (!is.null(S_inv)) {
          denom_S <- denom_S + lambda * abs(det(S)) * t(S_inv)
        }
      }
      S <- S * (numer_S / denom_S)^gamma
    } else {
      # model == "MM": Omega ≈ M M^T
      numer_M <- numer_weight %*% M
      denom_M <- denom_weight %*% M + eps
      gamma <- .muExponent(Beta)
      M <- M * (numer_M / denom_M)^gamma
    }

    # Ensure non-negativity
    M[M < eps] <- eps
    if (model == "MSM") S[S < eps] <- eps
  }

  RecError <- RecError[seq_len(iter)]
  RelChange <- RelChange[seq_len(iter)]

  list(
    M = M,
    S = S,
    RecError = RecError,
    RelChange = RelChange,
    converged = converged,
    iter = iter
  )
}

#' Compute Beta-divergence D_beta(X || Y)
#'
#' @param X Target matrix
#' @param Y Approximation matrix
#' @param beta Beta parameter
#' @return Scalar divergence value
#' @keywords internal
.betaDiv <- function(X, Y, beta) {
  eps <- .Machine$double.eps
  if (abs(beta - 2) < eps) {
    # Frobenius (Euclidean)
    sum((X - Y)^2) / 2
  } else if (abs(beta - 1) < eps) {
    # KL divergence
    sum(X * log(X / Y + eps) - X + Y)
  } else if (abs(beta) < eps) {
    # Itakura-Saito
    sum(X / Y - log(X / Y + eps) - 1)
  } else {
    # General Beta
    sum(X^beta / (beta * (beta - 1)) +
        Y^beta / beta -
        X * Y^(beta - 1) / (beta - 1))
  }
}

#' MU exponent for Beta-divergence
#'
#' Returns the exponent gamma for the multiplicative update rule.
#' gamma = 1 for Beta in [1,2], otherwise adjusted for convergence guarantee.
#'
#' @param beta Beta parameter
#' @return Exponent gamma
#' @references
#' Fevotte, C. & Idier, J. (2011). Neural Computation, 23(9).
#' @keywords internal
.muExponent <- function(beta) {
  if (beta >= 1 && beta <= 2) {
    1
  } else {
    1 / (3 - beta)
  }
}

library(symTensor)
library(rTensor)

context("symTensor")

test_that("isSymmetricTensor works for matrices", {
  A <- matrix(c(1, 2, 2, 3), 2, 2)
  expect_identical(isSymmetricTensor(A), TRUE)

  B <- matrix(c(1, 2, 3, 4), 2, 2)
  expect_identical(isSymmetricTensor(B), FALSE)
})

test_that("isSymmetricTensor works for 3-mode tensors", {
  arr <- array(0, dim = c(3, 3, 3))
  for (i in 1:3) for (j in 1:3) for (k in 1:3) {
    arr[i, j, k] <- i + j + k
  }
  expect_identical(isSymmetricTensor(as.tensor(arr)), TRUE)
})

test_that("Symmetrize works for matrices", {
  A <- matrix(c(1, 2, 3, 4), 2, 2)
  S <- Symmetrize(A)
  expect_identical(isSymmetricTensor(S), TRUE)
})

test_that("Symmetrize works for tensors", {
  arr <- array(rnorm(27), dim = c(3, 3, 3))
  S <- Symmetrize(as.tensor(arr))
  expect_identical(isSymmetricTensor(S), TRUE)
})

test_that("PageRank basic functionality", {
  A <- matrix(c(0, 1, 1, 0, 1, 0, 0, 1, 0, 1, 0, 1, 1, 0, 1, 0), 4, 4)
  pr <- PageRank(A)
  expect_lt(abs(sum(pr$vector) - 1), 1e-10)
  expect_identical(pr$converged, TRUE)
})

test_that("PageRank with personalization", {
  A <- matrix(c(0, 1, 1, 0, 1, 0, 0, 1, 0, 1, 0, 1, 1, 0, 1, 0), 4, 4)
  v <- c(1, 0, 0, 0)
  pr <- PageRank(A, personalization = v)
  expect_identical(pr$converged, TRUE)
  expect_identical(which.max(pr$vector), 1L)
})

test_that("LabelPropagation basic", {
  A <- matrix(0, 6, 6)
  A[1, 2] <- A[2, 1] <- 1
  A[1, 3] <- A[3, 1] <- 1
  A[2, 3] <- A[3, 2] <- 1
  A[4, 5] <- A[5, 4] <- 1
  A[4, 6] <- A[6, 4] <- 1
  A[5, 6] <- A[6, 5] <- 1
  A[3, 4] <- A[4, 3] <- 0.1
  seeds <- c("1" = 1, "6" = 2)
  result <- LabelPropagation(A, seeds)
  expect_identical(result$labels[1:3], c(1L, 1L, 1L))
  expect_identical(result$labels[4:6], c(2L, 2L, 2L))
})

test_that("symNMF MM Frobenius (Beta=2) error decreases", {
  set.seed(123)
  M_true <- matrix(0, 10, 2)
  M_true[1:5, 1] <- runif(5, 0.5, 1)
  M_true[6:10, 2] <- runif(5, 0.5, 1)
  Omega <- M_true %*% t(M_true)
  Omega <- Omega + matrix(runif(100, 0, 0.01), 10, 10)
  Omega <- (Omega + t(Omega)) / 2

  result <- symNMF(Omega, K = 2, model = "MM", Beta = 2, num.iter = 200)
  expect_gt(length(result$RecError), 1)
  expect_lte(tail(result$RecError, 1), result$RecError[1])
})

test_that("symNMF MSM Frobenius (Beta=2) error decreases", {
  set.seed(42)
  M_true <- matrix(runif(20, 0, 1), 10, 2)
  S_true <- matrix(c(2, 0.5, 0.5, 3), 2, 2)
  Omega <- M_true %*% S_true %*% t(M_true)

  result <- symNMF(Omega, K = 2, model = "MSM", Beta = 2, num.iter = 200)
  expect_gt(length(result$RecError), 1)
  expect_lte(tail(result$RecError, 1), result$RecError[1])
})

test_that("symNMF KL divergence (Beta=1) error decreases", {
  set.seed(42)
  M_true <- matrix(runif(20, 0.1, 1), 10, 2)
  Omega <- M_true %*% t(M_true)
  Omega <- Omega + matrix(runif(100, 0, 0.01), 10, 10)
  Omega <- (Omega + t(Omega)) / 2

  result <- symNMF(Omega, K = 2, model = "MM", Beta = 1, num.iter = 200)
  expect_gt(length(result$RecError), 1)
  expect_lte(tail(result$RecError, 1), result$RecError[1])
})

test_that("symNMF IS divergence (Beta=0) error decreases", {
  set.seed(42)
  M_true <- matrix(runif(20, 0.1, 1), 10, 2)
  Omega <- M_true %*% t(M_true)
  Omega <- Omega + matrix(runif(100, 0.01, 0.1), 10, 10)
  Omega <- (Omega + t(Omega)) / 2

  result <- symNMF(Omega, K = 2, model = "MM", Beta = 0, num.iter = 200)
  expect_gt(length(result$RecError), 1)
  expect_lte(tail(result$RecError, 1), result$RecError[1])
})

test_that("HigherOrderPower recovers rank-1 tensor", {
  set.seed(42)
  n <- 5
  v1 <- rnorm(n)
  v1 <- v1 / sqrt(sum(v1^2))
  lam <- 3.0
  arr <- lam * outer(outer(v1, v1), v1)
  tnsr <- as.tensor(arr)

  result <- HigherOrderPower(tnsr, R = 1)
  expect_lt(abs(abs(result$lambdas[1]) - 3.0), 0.1)
  cosine <- abs(sum(result$vectors[, 1] * v1))
  expect_gt(cosine, 0.95)
})

test_that("TOPHITS runs without error", {
  set.seed(42)
  arr <- array(abs(rnorm(125)), dim = c(5, 5, 5))
  tnsr <- as.tensor(arr)
  result <- TOPHITS(tnsr, R = 2)
  expect_identical(length(result$rankings), 3L)
  expect_identical(length(result$rankings[[1]]), 5L)
})

#' Symmetrize a matrix or tensor
#'
#' For a matrix, computes (A + A^T) / 2.
#' For a tensor, averages over all permutations of indices.
#'
#' @param x A matrix or rTensor Tensor object (must have equal modes)
#' @return Symmetrized matrix or Tensor
#' @export
#' @examples
#' A <- matrix(c(1, 2, 3, 4), 2, 2)
#' Symmetrize(A)
#'
#' library(rTensor)
#' arr <- array(rnorm(27), dim = c(3, 3, 3))
#' S <- Symmetrize(as.tensor(arr))
#' isSymmetricTensor(S)  # TRUE
Symmetrize <- function(x) {
  if (is.matrix(x)) {
    if (nrow(x) != ncol(x)) stop("Matrix must be square")
    return((x + t(x)) / 2)
  }
  if (inherits(x, "Tensor")) {
    modes <- x@modes
    if (length(unique(modes)) != 1) {
      stop("All modes must be equal for symmetrization")
    }
    d <- x@num_modes
    if (d == 1) return(x)
    arr <- x@data
    perms <- .allPermutations(d)
    result <- array(0, dim = modes)
    for (p in perms) {
      result <- result + aperm(arr, p)
    }
    result <- result / length(perms)
    return(as.tensor(result))
  }
  stop("x must be a matrix or rTensor Tensor object")
}

#' Check if a matrix or tensor is symmetric
#'
#' For a matrix, checks if A[i,j] == A[j,i] for all i,j.
#' For a tensor, checks if the tensor is invariant under all permutations
#' of its indices (i.e., fully symmetric).
#'
#' @param x A matrix or rTensor Tensor object
#' @param tol Numeric tolerance for comparison (default: 1e-10)
#' @return Logical indicating whether x is symmetric
#' @export
#' @examples
#' # Matrix
#' A <- matrix(c(1,2,2,3), 2, 2)
#' isSymmetricTensor(A)  # TRUE
#'
#' # 3-mode symmetric tensor
#' library(rTensor)
#' arr <- array(0, dim = c(3, 3, 3))
#' for (i in 1:3) for (j in 1:3) for (k in 1:3) {
#'   arr[i, j, k] <- i + j + k
#' }
#' isSymmetricTensor(as.tensor(arr))  # TRUE
isSymmetricTensor <- function(x, tol = 1e-10) {
  if (is.matrix(x)) {
    if (nrow(x) != ncol(x)) return(FALSE)
    return(all(abs(x - t(x)) <= tol))
  }
  if (inherits(x, "Tensor")) {
    modes <- x@modes
    # All modes must be equal for full symmetry
    if (length(unique(modes)) != 1) return(FALSE)
    d <- x@num_modes
    if (d == 1) return(TRUE)
    arr <- x@data
    # Check all pairwise transpositions
    perms <- .allPermutations(d)
    for (p in perms) {
      if (identical(p, seq_len(d))) next
      if (any(abs(arr - aperm(arr, p)) > tol)) return(FALSE)
    }
    return(TRUE)
  }
  stop("x must be a matrix or rTensor Tensor object")
}

#' Generate all permutations of 1:n
#' @param n Integer
#' @return List of integer vectors
#' @keywords internal
.allPermutations <- function(n) {
  if (n == 1) return(list(1L))
  if (n == 2) return(list(1:2, 2:1))
  result <- list()
  sub <- .allPermutations(n - 1L)
  for (perm in sub) {
    for (pos in seq_len(n)) {
      new_perm <- append(perm, n, after = pos - 1)
      result <- c(result, list(new_perm))
    }
  }
  result
}

#' Label Propagation via PageRank
#'
#' Semi-supervised label propagation on a graph. Uses Personalized PageRank
#' internally to propagate labels from seed nodes to the rest of the graph.
#'
#' @param A Square non-negative adjacency/weight matrix (n x n)
#' @param seeds Named integer vector or factor. Names correspond to node indices
#'   (1-based), values are the class labels. Unlabeled nodes will be classified.
#' @param damping Damping factor passed to \code{\link{PageRank}} (default: 0.85)
#' @param max_iter Maximum iterations for PageRank (default: 1000)
#' @param tol Convergence tolerance for PageRank (default: 1e-8)
#' @return A list with components:
#'   \describe{
#'     \item{labels}{Integer vector of predicted labels for all nodes}
#'     \item{scores}{Matrix (n x K) of PageRank-based scores per class}
#'     \item{classes}{Unique class labels}
#'   }
#' @export
#' @examples
#' # 6-node graph with 2 communities
#' A <- matrix(0, 6, 6)
#' A[1,2] <- A[2,1] <- 1; A[1,3] <- A[3,1] <- 1; A[2,3] <- A[3,2] <- 1
#' A[4,5] <- A[5,4] <- 1; A[4,6] <- A[6,4] <- 1; A[5,6] <- A[6,5] <- 1
#' A[3,4] <- A[4,3] <- 0.5  # weak bridge
#' seeds <- c("1" = 1, "6" = 2)  # node 1 -> class 1, node 6 -> class 2
#' result <- LabelPropagation(A, seeds)
#' result$labels
LabelPropagation <- function(A, seeds, damping = 0.85,
                             max_iter = 1000L, tol = 1e-8) {
  if (!is.matrix(A)) stop("A must be a matrix")
  n <- nrow(A)

  seed_indices <- as.integer(names(seeds))
  seed_labels <- as.integer(seeds)
  classes <- sort(unique(seed_labels))
  K <- length(classes)

  # Run Personalized PageRank for each class
  scores <- matrix(0, nrow = n, ncol = K)
  colnames(scores) <- as.character(classes)

  for (k in seq_len(K)) {
    # Personalization: uniform over seed nodes of class k
    v <- rep(0, n)
    class_seeds <- seed_indices[seed_labels == classes[k]]
    v[class_seeds] <- 1 / length(class_seeds)

    pr <- PageRank(A, damping = damping, personalization = v,
                   max_iter = max_iter, tol = tol)
    scores[, k] <- pr$vector
  }

  # Assign each node to the class with highest score
  labels <- classes[apply(scores, 1, which.max)]

  list(labels = labels, scores = scores, classes = classes)
}

#' Higher-order Power Method for Symmetric Tensor Decomposition
#'
#' Decomposes a symmetric tensor into a sum of rank-1 symmetric terms
#' using the higher-order power method (S-HOPM) with deflation.
#'
#' For a d-th order symmetric tensor \eqn{T \in R^{n \times n \times \cdots \times n}},
#' finds \eqn{T \approx \sum_{r=1}^{R} \lambda_r v_r \otimes v_r \otimes \cdots \otimes v_r}.
#'
#' @param x A symmetric rTensor Tensor object (all modes must be equal)
#' @param R Number of rank-1 components to extract (default: 1)
#' @param max_iter Maximum iterations per component (default: 1000)
#' @param tol Convergence tolerance (default: 1e-8)
#' @param init Initialization method: \code{"random"} (default) or \code{"svd"}
#' @return A list with components:
#'   \describe{
#'     \item{lambdas}{Numeric vector of length R with eigenvalues}
#'     \item{vectors}{Matrix (n x R) where each column is a unit eigenvector}
#'     \item{residual}{Residual tensor after deflation}
#'     \item{converged}{Logical vector of length R}
#'     \item{iters}{Integer vector of iterations per component}
#'   }
#' @references
#' Kolda, T. G., & Mayo, J. R. (2011).
#' Shifted Power Method for Computing Tensor Eigenpairs.
#' SIAM Journal on Matrix Analysis and Applications.
#'
#' De Lathauwer, L., De Moor, B., & Vandewalle, J. (2000).
#' On the Best Rank-1 and Rank-(R1, R2, ..., RN) Approximation of
#' Higher-Order Tensors. SIAM Journal on Matrix Analysis and Applications.
#' @export
#' @examples
#' library(rTensor)
#' # Create a symmetric 3-mode tensor
#' n <- 5
#' v1 <- rnorm(n); v1 <- v1 / sqrt(sum(v1^2))
#' # T = 3.0 * v1 o v1 o v1
#' arr <- 3.0 * outer(outer(v1, v1), v1)
#' tnsr <- as.tensor(arr)
#' result <- HigherOrderPower(tnsr, R = 1)
#' result$lambdas  # should be close to 3.0
HigherOrderPower <- function(x, R = 1L, max_iter = 1000L, tol = 1e-8,
                             init = c("random", "svd")) {
  init <- match.arg(init)

  if (!inherits(x, "Tensor")) {
    stop("x must be an rTensor Tensor object")
  }
  modes <- x@modes
  if (length(unique(modes)) != 1) {
    stop("All modes must be equal (symmetric tensor required)")
  }
  n <- modes[1]
  d <- x@num_modes

  lambdas <- numeric(R)
  vectors <- matrix(0, n, R)
  converged_vec <- logical(R)
  iters_vec <- integer(R)

  residual <- x@data

  for (r in seq_len(R)) {
    # Initialize
    if (init == "svd" && d >= 2) {
      # Use leading left singular vector of mode-1 unfolding
      unf <- matrix(residual, n, n^(d - 1))
      v <- svd(unf, nu = 1, nv = 0)$u[, 1]
    } else {
      v <- rnorm(n)
    }
    v <- v / sqrt(sum(v^2))

    conv <- FALSE
    for (iter in seq_len(max_iter)) {
      # Contract residual tensor with v along all modes except the first
      # T(I, v, v, ..., v)
      w <- .tensorVectorContract(residual, v, d)
      lambda <- sum(w * v)
      w_norm <- sqrt(sum(w^2))

      if (w_norm < .Machine$double.eps) {
        v_new <- v
      } else {
        v_new <- w / w_norm
      }

      # Check convergence (allow sign flip)
      change <- min(sqrt(sum((v_new - v)^2)), sqrt(sum((v_new + v)^2)))
      if (change < tol) {
        conv <- TRUE
        v <- v_new
        break
      }
      v <- v_new
    }

    # Final eigenvalue
    lambda <- .tensorVectorContractAll(residual, v, d)

    lambdas[r] <- lambda
    vectors[, r] <- v
    converged_vec[r] <- conv
    iters_vec[r] <- iter

    # Deflation: subtract rank-1 component
    rank1 <- lambda * .outerRepeat(v, d)
    residual <- residual - rank1
  }

  list(
    lambdas = lambdas,
    vectors = vectors,
    residual = as.tensor(residual),
    converged = converged_vec,
    iters = iters_vec
  )
}

#' Contract a tensor with a vector along all modes except the first
#' @param arr Array of dimension (n, n, ..., n)
#' @param v Numeric vector of length n
#' @param d Number of modes
#' @return Numeric vector of length n
#' @keywords internal
.tensorVectorContract <- function(arr, v, d) {
  # Contract modes 2, 3, ..., d with v
  result <- arr
  for (m in seq(d, 2, by = -1)) {
    # Contract mode m with v
    result <- .contractMode(result, v, m)
  }
  as.vector(result)
}

#' Full contraction of tensor with vector along all modes
#' @keywords internal
.tensorVectorContractAll <- function(arr, v, d) {
  result <- arr
  for (m in seq(d, 1, by = -1)) {
    result <- .contractMode(result, v, m)
  }
  as.numeric(result)
}

#' Contract an array along a specific mode with a vector
#' @param arr Multi-dimensional array
#' @param v Vector
#' @param mode Mode to contract
#' @return Array with one fewer dimension
#' @keywords internal
.contractMode <- function(arr, v, mode) {
  dims <- dim(arr)
  if (is.null(dims)) {
    # Already a vector
    return(sum(arr * v))
  }
  d <- length(dims)
  n <- dims[mode]

  # Permute so that the target mode is the last
  perm <- c(seq_len(d)[-mode], mode)
  arr_perm <- aperm(arr, perm)

  # Reshape to matrix: (product of other dims) x n
  other_dims <- dims[-mode]
  arr_mat <- matrix(arr_perm, prod(other_dims), n)

  # Multiply
  result <- arr_mat %*% v

  # Reshape back
  if (length(other_dims) == 1) {
    return(as.vector(result))
  }
  array(result, dim = other_dims)
}

#' Compute repeated outer product of a vector
#' @param v Numeric vector
#' @param d Number of times to take outer product
#' @return d-dimensional array
#' @keywords internal
.outerRepeat <- function(v, d) {
  n <- length(v)
  if (d == 1) return(v)
  result <- outer(v, v)
  if (d == 2) return(result)
  for (i in 3:d) {
    result <- outer(result, v)
  }
  result
}

#' TOPHITS: Tensor HITS Algorithm
#'
#' Computes hub and authority scores for higher-order link analysis using
#' Tucker decomposition of a tensor. Generalization of HITS (Hyperlink-Induced
#' Topic Search) to tensors.
#'
#' For a 3-mode tensor \eqn{T \in R^{I \times J \times K}},
#' TOPHITS computes Tucker decomposition \eqn{T \approx C \times_1 U_1 \times_2 U_2 \times_3 U_3}
#' and derives hub/authority scores from the factor matrices.
#'
#' For a symmetric tensor (all modes equal), a single set of scores is returned.
#'
#' @param x An rTensor Tensor object (2-mode or 3-mode)
#' @param R Integer or integer vector specifying the number of components per mode.
#'   If scalar, same rank is used for all modes.
#' @param max_iter Maximum iterations for Tucker decomposition (default: 100)
#' @param tol Convergence tolerance (default: 1e-8)
#' @return A list with components:
#'   \describe{
#'     \item{scores}{List of score matrices, one per mode (each n_i x R_i).
#'       For symmetric tensors, a single matrix.}
#'     \item{core}{Core tensor from Tucker decomposition}
#'     \item{rankings}{List of integer vectors giving node rankings per mode
#'       (ordered by dominant score)}
#'     \item{decomp}{Full Tucker decomposition result from rTensor}
#'   }
#' @references
#' Kolda, T. G., & Bader, B. W. (2006).
#' The TOPHITS Model for Higher-Order Web Link Analysis.
#' Workshop on Link Analysis, Counterterrorism and Security.
#' @export
#' @examples
#' library(rTensor)
#' # Create a 3-mode tensor
#' arr <- array(abs(rnorm(125)), dim = c(5, 5, 5))
#' tnsr <- as.tensor(arr)
#' result <- TOPHITS(tnsr, R = 2)
#' result$rankings
TOPHITS <- function(x, R = 2L, max_iter = 100L, tol = 1e-8) {
  if (!inherits(x, "Tensor")) {
    stop("x must be an rTensor Tensor object")
  }

  d <- x@num_modes
  modes <- x@modes

  # Set ranks per mode
  if (length(R) == 1) {
    ranks <- rep(R, d)
  } else {
    if (length(R) != d) stop("R must be scalar or length equal to num_modes")
    ranks <- R
  }
  # Ensure ranks don't exceed mode dimensions
  ranks <- pmin(ranks, modes)

  # Tucker decomposition (HOOI via rTensor)
  tuck <- rTensor::tucker(x, ranks = ranks, max_iter = max_iter, tol = tol)

  # Extract factor matrices (scores)
  U_list <- tuck$U
  core <- tuck$Z

  # Compute scores: |U_i| weighted by core tensor norms along each mode
  # For each mode, the score of node j is the norm of the j-th row of U_i
  # weighted by corresponding core slices
  scores <- list()
  rankings <- list()

  is_symmetric <- length(unique(modes)) == 1

  for (m in seq_len(d)) {
    U <- U_list[[m]]
    # Primary score: magnitude of each row in factor matrix
    # Weight by squared singular values (core tensor energy along this mode)
    core_unfold <- rTensor::k_unfold(core, m)@data
    weights <- rowSums(core_unfold^2)
    weights <- weights / sum(weights)

    # Weighted score for each node
    score_mat <- sweep(U^2, 2, weights, "*")
    node_scores <- rowSums(score_mat)

    scores[[m]] <- U
    rankings[[m]] <- order(node_scores, decreasing = TRUE)
  }

  if (is_symmetric) {
    names(scores) <- paste0("mode", seq_len(d))
    names(rankings) <- paste0("mode", seq_len(d))
  } else {
    names(scores) <- paste0("mode", seq_len(d))
    names(rankings) <- paste0("mode", seq_len(d))
  }

  list(
    scores = scores,
    core = core,
    rankings = rankings,
    decomp = tuck
  )
}

#' (Personalized) PageRank
#'
#' Computes the PageRank vector for a given adjacency or transition matrix
#' using power iteration.
#'
#' @param A Square non-negative matrix (adjacency or weight matrix).
#'   Columns are normalized internally to form a column-stochastic matrix.
#' @param damping Damping factor (default: 0.85). Probability of following a link.
#' @param personalization Optional personalization vector of length \code{nrow(A)}.
#'   If NULL, uniform distribution is used (standard PageRank).
#'   Personalized PageRank biases the random jumps toward this distribution.
#' @param max_iter Maximum number of power iterations (default: 1000)
#' @param tol Convergence tolerance on L1 norm change (default: 1e-8)
#' @return A list with components:
#'   \describe{
#'     \item{vector}{PageRank vector (sums to 1)}
#'     \item{iter}{Number of iterations until convergence}
#'     \item{converged}{Logical, whether the algorithm converged}
#'   }
#' @export
#' @examples
#' # Simple web graph
#' A <- matrix(c(0,1,1,0, 1,0,0,1, 0,1,0,1, 1,0,1,0), 4, 4)
#' pr <- PageRank(A)
#' pr$vector
PageRank <- function(A, damping = 0.85, personalization = NULL,
                     max_iter = 1000L, tol = 1e-8) {
  if (!is.matrix(A)) stop("A must be a matrix")
  n <- nrow(A)
  if (n != ncol(A)) stop("A must be square")

  # Personalization vector
  if (is.null(personalization)) {
    v <- rep(1 / n, n)
  } else {
    if (length(personalization) != n) {
      stop("personalization must have length nrow(A)")
    }
    v <- personalization / sum(personalization)
  }

  # Column-normalize A to get transition matrix
  col_sums <- colSums(A)
  # Handle dangling nodes (columns with zero sum)
  dangling <- which(col_sums == 0)
  col_sums[dangling] <- 1  # avoid division by zero
  M <- sweep(A, 2, col_sums, "/")

  # Power iteration: pi = d * M %*% pi + (1-d) * v + d * dangling_contribution
  pi <- rep(1 / n, n)
  converged <- FALSE

  for (iter in seq_len(max_iter)) {
    pi_new <- damping * (M %*% pi) + (1 - damping) * v
    # Dangling node contribution: redistribute their mass
    if (length(dangling) > 0) {
      dangling_mass <- damping * sum(pi[dangling])
      pi_new <- pi_new + dangling_mass * v
    }
    pi_new <- as.vector(pi_new)
    pi_new <- pi_new / sum(pi_new)

    if (sum(abs(pi_new - pi)) < tol) {
      converged <- TRUE
      pi <- pi_new
      break
    }
    pi <- pi_new
  }

  list(vector = pi, iter = iter, converged = converged)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/symNMF.R
\name{.betaDiv}
\alias{.betaDiv}
\title{Compute Beta-divergence D_beta(X || Y)}
\usage{
.betaDiv(X, Y, beta)
}
\arguments{
\item{X}{Target matrix}

\item{Y}{Approximation matrix}

\item{beta}{Beta parameter}
}
\value{
Scalar divergence value
}
\description{
Compute Beta-divergence D_beta(X || Y)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HigherOrderPower.R
\name{.tensorVectorContractAll}
\alias{.tensorVectorContractAll}
\title{Full contraction of tensor with vector along all modes}
\usage{
.tensorVectorContractAll(arr, v, d)
}
\description{
Full contraction of tensor with vector along all modes
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PageRank.R
\name{PageRank}
\alias{PageRank}
\title{(Personalized) PageRank}
\usage{
PageRank(
  A,
  damping = 0.85,
  personalization = NULL,
  max_iter = 1000L,
  tol = 1e-08
)
}
\arguments{
\item{A}{Square non-negative matrix (adjacency or weight matrix).
Columns are normalized internally to form a column-stochastic matrix.}

\item{damping}{Damping factor (default: 0.85). Probability of following a link.}

\item{personalization}{Optional personalization vector of length \code{nrow(A)}.
If NULL, uniform distribution is used (standard PageRank).
Personalized PageRank biases the random jumps toward this distribution.}

\item{max_iter}{Maximum number of power iterations (default: 1000)}

\item{tol}{Convergence tolerance on L1 norm change (default: 1e-8)}
}
\value{
A list with components:
  \describe{
    \item{vector}{PageRank vector (sums to 1)}
    \item{iter}{Number of iterations until convergence}
    \item{converged}{Logical, whether the algorithm converged}
  }
}
\description{
Computes the PageRank vector for a given adjacency or transition matrix
using power iteration.
}
\examples{
# Simple web graph
A <- matrix(c(0,1,1,0, 1,0,0,1, 0,1,0,1, 1,0,1,0), 4, 4)
pr <- PageRank(A)
pr$vector
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/isSymmetric.R
\name{isSymmetricTensor}
\alias{isSymmetricTensor}
\title{Check if a matrix or tensor is symmetric}
\usage{
isSymmetricTensor(x, tol = 1e-10)
}
\arguments{
\item{x}{A matrix or rTensor Tensor object}

\item{tol}{Numeric tolerance for comparison (default: 1e-10)}
}
\value{
Logical indicating whether x is symmetric
}
\description{
For a matrix, checks if A[i,j] == A[j,i] for all i,j.
For a tensor, checks if the tensor is invariant under all permutations
of its indices (i.e., fully symmetric).
}
\examples{
# Matrix
A <- matrix(c(1,2,2,3), 2, 2)
isSymmetricTensor(A)  # TRUE

# 3-mode symmetric tensor
library(rTensor)
arr <- array(0, dim = c(3, 3, 3))
for (i in 1:3) for (j in 1:3) for (k in 1:3) {
  arr[i, j, k] <- i + j + k
}
isSymmetricTensor(as.tensor(arr))  # TRUE
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/symNMF.R
\name{.muExponent}
\alias{.muExponent}
\title{MU exponent for Beta-divergence}
\usage{
.muExponent(beta)
}
\arguments{
\item{beta}{Beta parameter}
}
\value{
Exponent gamma
}
\description{
Returns the exponent gamma for the multiplicative update rule.
gamma = 1 for Beta in [1,2], otherwise adjusted for convergence guarantee.
}
\references{
Fevotte, C. & Idier, J. (2011). Neural Computation, 23(9).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HigherOrderPower.R
\name{.tensorVectorContract}
\alias{.tensorVectorContract}
\title{Contract a tensor with a vector along all modes except the first}
\usage{
.tensorVectorContract(arr, v, d)
}
\arguments{
\item{arr}{Array of dimension (n, n, ..., n)}

\item{v}{Numeric vector of length n}

\item{d}{Number of modes}
}
\value{
Numeric vector of length n
}
\description{
Contract a tensor with a vector along all modes except the first
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LabelPropagation.R
\name{LabelPropagation}
\alias{LabelPropagation}
\title{Label Propagation via PageRank}
\usage{
LabelPropagation(A, seeds, damping = 0.85, max_iter = 1000L, tol = 1e-08)
}
\arguments{
\item{A}{Square non-negative adjacency/weight matrix (n x n)}

\item{seeds}{Named integer vector or factor. Names correspond to node indices
(1-based), values are the class labels. Unlabeled nodes will be classified.}

\item{damping}{Damping factor passed to \code{\link{PageRank}} (default: 0.85)}

\item{max_iter}{Maximum iterations for PageRank (default: 1000)}

\item{tol}{Convergence tolerance for PageRank (default: 1e-8)}
}
\value{
A list with components:
  \describe{
    \item{labels}{Integer vector of predicted labels for all nodes}
    \item{scores}{Matrix (n x K) of PageRank-based scores per class}
    \item{classes}{Unique class labels}
  }
}
\description{
Semi-supervised label propagation on a graph. Uses Personalized PageRank
internally to propagate labels from seed nodes to the rest of the graph.
}
\examples{
# 6-node graph with 2 communities
A <- matrix(0, 6, 6)
A[1,2] <- A[2,1] <- 1; A[1,3] <- A[3,1] <- 1; A[2,3] <- A[3,2] <- 1
A[4,5] <- A[5,4] <- 1; A[4,6] <- A[6,4] <- 1; A[5,6] <- A[6,5] <- 1
A[3,4] <- A[4,3] <- 0.5  # weak bridge
seeds <- c("1" = 1, "6" = 2)  # node 1 -> class 1, node 6 -> class 2
result <- LabelPropagation(A, seeds)
result$labels
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HigherOrderPower.R
\name{.outerRepeat}
\alias{.outerRepeat}
\title{Compute repeated outer product of a vector}
\usage{
.outerRepeat(v, d)
}
\arguments{
\item{v}{Numeric vector}

\item{d}{Number of times to take outer product}
}
\value{
d-dimensional array
}
\description{
Compute repeated outer product of a vector
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TOPHITS.R
\name{TOPHITS}
\alias{TOPHITS}
\title{TOPHITS: Tensor HITS Algorithm}
\usage{
TOPHITS(x, R = 2L, max_iter = 100L, tol = 1e-08)
}
\arguments{
\item{x}{An rTensor Tensor object (2-mode or 3-mode)}

\item{R}{Integer or integer vector specifying the number of components per mode.
If scalar, same rank is used for all modes.}

\item{max_iter}{Maximum iterations for Tucker decomposition (default: 100)}

\item{tol}{Convergence tolerance (default: 1e-8)}
}
\value{
A list with components:
  \describe{
    \item{scores}{List of score matrices, one per mode (each n_i x R_i).
      For symmetric tensors, a single matrix.}
    \item{core}{Core tensor from Tucker decomposition}
    \item{rankings}{List of integer vectors giving node rankings per mode
      (ordered by dominant score)}
    \item{decomp}{Full Tucker decomposition result from rTensor}
  }
}
\description{
Computes hub and authority scores for higher-order link analysis using
Tucker decomposition of a tensor. Generalization of HITS (Hyperlink-Induced
Topic Search) to tensors.
}
\details{
For a 3-mode tensor \eqn{T \in R^{I \times J \times K}},
TOPHITS computes Tucker decomposition \eqn{T \approx C \times_1 U_1 \times_2 U_2 \times_3 U_3}
and derives hub/authority scores from the factor matrices.

For a symmetric tensor (all modes equal), a single set of scores is returned.
}
\examples{
library(rTensor)
# Create a 3-mode tensor
arr <- array(abs(rnorm(125)), dim = c(5, 5, 5))
tnsr <- as.tensor(arr)
result <- TOPHITS(tnsr, R = 2)
result$rankings
}
\references{
Kolda, T. G., & Bader, B. W. (2006).
The TOPHITS Model for Higher-Order Web Link Analysis.
Workshop on Link Analysis, Counterterrorism and Security.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Symmetrize.R
\name{Symmetrize}
\alias{Symmetrize}
\title{Symmetrize a matrix or tensor}
\usage{
Symmetrize(x)
}
\arguments{
\item{x}{A matrix or rTensor Tensor object (must have equal modes)}
}
\value{
Symmetrized matrix or Tensor
}
\description{
For a matrix, computes (A + A^T) / 2.
For a tensor, averages over all permutations of indices.
}
\examples{
A <- matrix(c(1, 2, 3, 4), 2, 2)
Symmetrize(A)

library(rTensor)
arr <- array(rnorm(27), dim = c(3, 3, 3))
S <- Symmetrize(as.tensor(arr))
isSymmetricTensor(S)  # TRUE
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/symNMF.R
\name{symNMF}
\alias{symNMF}
\title{Symmetric Non-negative Matrix Factorization (symNMF)}
\usage{
symNMF(
  Omega,
  K,
  model = c("MSM", "MM"),
  Beta = 2,
  initM = NULL,
  initS = NULL,
  lambda = 0,
  num.iter = 100L,
  thr = 1e-10,
  verbose = FALSE
)
}
\arguments{
\item{Omega}{Symmetric non-negative matrix (N x N)}

\item{K}{Number of components (rank)}

\item{model}{Character, either \code{"MSM"} (default) or \code{"MM"}}

\item{Beta}{Beta parameter for Beta-divergence (default: 2).
2 = Frobenius, 1 = KL, 0 = IS.}

\item{initM}{Initial M matrix (N x K). If NULL, random non-negative initialization.}

\item{initS}{Initial S matrix (K x K). If NULL, random non-negative initialization.
Ignored when \code{model = "MM"}.}

\item{lambda}{Regularization parameter for \eqn{|\det(S)|} penalty (default: 0).
Only used with \code{model = "MSM"}.}

\item{num.iter}{Maximum number of iterations (default: 100)}

\item{thr}{Convergence threshold on relative change (default: 1e-10)}

\item{verbose}{Logical, print iteration info (default: FALSE)}
}
\value{
A list with components:
  \describe{
    \item{M}{Factor matrix (N x K)}
    \item{S}{Coefficient matrix (K x K). Identity matrix when \code{model = "MM"}.}
    \item{RecError}{Vector of reconstruction errors at each iteration}
    \item{RelChange}{Vector of relative changes at each iteration}
    \item{converged}{Logical}
    \item{iter}{Number of iterations performed}
  }
}
\description{
Decomposes a symmetric non-negative matrix \eqn{\Omega \approx M S M^\top}
using multiplicative update (MU) rules with Beta-divergence.
}
\details{
Two models are supported:
\itemize{
  \item \strong{MSM}: \eqn{\Omega \approx M S M^\top} where M is N x K and S is K x K
  \item \strong{MM}: \eqn{\Omega \approx M M^\top} where M is N x K (special case with S = I)
}

The loss function is the Beta-divergence \eqn{D_\beta(\Omega \| \hat{\Omega})}:
\itemize{
  \item \eqn{\beta = 2}: Frobenius (Euclidean) distance
  \item \eqn{\beta = 1}: Kullback-Leibler divergence
  \item \eqn{\beta = 0}: Itakura-Saito divergence
  \item Other values of \eqn{\beta} interpolate/extrapolate these cases
}
}
\examples{
set.seed(42)
M_true <- matrix(c(rep(c(1,0), 5), rep(c(0,1), 5)), 10, 2)
Omega <- M_true \%*\% t(M_true) + matrix(runif(100, 0, 0.1), 10, 10)
Omega <- (Omega + t(Omega)) / 2

# Frobenius (Beta=2, default)
res_fro <- symNMF(Omega, K = 2, Beta = 2)

# KL divergence (Beta=1)
res_kl <- symNMF(Omega, K = 2, Beta = 1)

# Itakura-Saito (Beta=0)
res_is <- symNMF(Omega, K = 2, Beta = 0)
}
\references{
Kuang, D., Park, H., & Ding, C. H. Q. (2012).
Symmetric Nonnegative Matrix Factorization for Graph Clustering.
SDM 2012.

Fevotte, C. & Idier, J. (2011).
Algorithms for Nonnegative Matrix Factorization with the
Beta-Divergence. Neural Computation, 23(9).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/isSymmetric.R
\name{.allPermutations}
\alias{.allPermutations}
\title{Generate all permutations of 1:n}
\usage{
.allPermutations(n)
}
\arguments{
\item{n}{Integer}
}
\value{
List of integer vectors
}
\description{
Generate all permutations of 1:n
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HigherOrderPower.R
\name{HigherOrderPower}
\alias{HigherOrderPower}
\title{Higher-order Power Method for Symmetric Tensor Decomposition}
\usage{
HigherOrderPower(
  x,
  R = 1L,
  max_iter = 1000L,
  tol = 1e-08,
  init = c("random", "svd")
)
}
\arguments{
\item{x}{A symmetric rTensor Tensor object (all modes must be equal)}

\item{R}{Number of rank-1 components to extract (default: 1)}

\item{max_iter}{Maximum iterations per component (default: 1000)}

\item{tol}{Convergence tolerance (default: 1e-8)}

\item{init}{Initialization method: \code{"random"} (default) or \code{"svd"}}
}
\value{
A list with components:
  \describe{
    \item{lambdas}{Numeric vector of length R with eigenvalues}
    \item{vectors}{Matrix (n x R) where each column is a unit eigenvector}
    \item{residual}{Residual tensor after deflation}
    \item{converged}{Logical vector of length R}
    \item{iters}{Integer vector of iterations per component}
  }
}
\description{
Decomposes a symmetric tensor into a sum of rank-1 symmetric terms
using the higher-order power method (S-HOPM) with deflation.
}
\details{
For a d-th order symmetric tensor \eqn{T \in R^{n \times n \times \cdots \times n}},
finds \eqn{T \approx \sum_{r=1}^{R} \lambda_r v_r \otimes v_r \otimes \cdots \otimes v_r}.
}
\examples{
library(rTensor)
# Create a symmetric 3-mode tensor
n <- 5
v1 <- rnorm(n); v1 <- v1 / sqrt(sum(v1^2))
# T = 3.0 * v1 o v1 o v1
arr <- 3.0 * outer(outer(v1, v1), v1)
tnsr <- as.tensor(arr)
result <- HigherOrderPower(tnsr, R = 1)
result$lambdas  # should be close to 3.0
}
\references{
Kolda, T. G., & Mayo, J. R. (2011).
Shifted Power Method for Computing Tensor Eigenpairs.
SIAM Journal on Matrix Analysis and Applications.

De Lathauwer, L., De Moor, B., & Vandewalle, J. (2000).
On the Best Rank-1 and Rank-(R1, R2, ..., RN) Approximation of
Higher-Order Tensors. SIAM Journal on Matrix Analysis and Applications.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HigherOrderPower.R
\name{.contractMode}
\alias{.contractMode}
\title{Contract an array along a specific mode with a vector}
\usage{
.contractMode(arr, v, mode)
}
\arguments{
\item{arr}{Multi-dimensional array}

\item{v}{Vector}

\item{mode}{Mode to contract}
}
\value{
Array with one fewer dimension
}
\description{
Contract an array along a specific mode with a vector
}
\keyword{internal}
